/*
 * @author Elena Hervalejo
 */
package org.hdtourist.model;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

import org.hdtourist.utils.StringUtils;

/**
 * Consulta.
 * Objeto que almacena la configuracin de una consulta
 */
public class Consulta {

	/**
	 * Tipo de consulta
	 */
	public enum TipoConsulta {

		/** Tipo AND. */
		AND,
		/** Tipo OR. */
		OR,
		/** Tipo bsqueda, equivalente a OR. */
		SEARCH
	}

	/** Consultas en forma de tubera. */
	private HashSet<ConsultaTuberia> consultasTuberia;

	/** El texto buscado en una consulta de tpo SEARCH. */
	private String texto = null;

	/** El tipo de consulta. */
	private TipoConsulta tipo;

	/** Los triples. */
	private List<Triple> triples;

	/**
	 * Instancia una nueva consulta.
	 */
	public Consulta() {
		this.consultasTuberia = new HashSet<ConsultaTuberia>();
		this.triples = new ArrayList<Triple>();
	}

	/**
	 * Obtiene la lista de consultas tubera.
	 *
	 * @return consultas tuberia
	 */
	public HashSet<ConsultaTuberia> getConsultasTuberia() {
		return this.consultasTuberia;
	}

	/**
	 * Obtiene el texto de bsqueda.
	 *
	 * @return texto de bsqueda
	 */
	public String getTexto() {
		return StringUtils.isNotEmpty(this.texto) ? this.texto : null;
	}

	/**
	 * Fija el tipo de consulta.
	 *
	 * @return tipo de consulta
	 */
	public TipoConsulta getTipo() {
		return this.tipo;
	}

	/**
	 * Obtiene los triples.
	 *
	 * @return los triples
	 */
	public List<Triple> getTriples() {
		return this.triples;
	}

	/**
	 * Indica si el tipo de consulta es AND
	 *
	 * @return true, si el tipo de consulta es AND
	 */
	public boolean isTipoConsultaAND() {
		return this.tipo == TipoConsulta.AND;
	}

	/**
	 * Indica si el tipo de consulta es OR
	 *
	 * @return true, si el tipo de consulta es OR
	 */
	public boolean isTipoConsultaOR() {
		return this.tipo == TipoConsulta.OR || this.isTipoConsultaSEARCH();
	}

	/**
	 * Indica si el tipo de consulta es SEARCH
	 *
	 * @return true, si el tipo de consulta es SERACH
	 */
	public boolean isTipoConsultaSEARCH() {
		return this.tipo == TipoConsulta.SEARCH;
	}

	/**
	 * Fija las consultas tubera
	 *
	 * @param consultasTuberia
	 *            las nuevas consultas tubera
	 */
	public void setConsultasTuberia(HashSet<ConsultaTuberia> consultasTuberia) {
		this.consultasTuberia = consultasTuberia;
	}

	/**
	 * Fija el texto de bsqueda
	 *
	 * @param texto
	 *            el nuevo texto de bsqueda
	 */
	public void setTexto(String texto) {
		this.texto = texto;
	}

	/**
	 * Fija el tipo consulta and.
	 */
	public void setTipoConsultaAND() {
		this.tipo = TipoConsulta.AND;
	}

	/**
	 * Fija el tipo consulta or.
	 */
	public void setTipoConsultaOR() {
		this.tipo = TipoConsulta.OR;
	}

	/**
	 * Fija el tipo consulta search.
	 */
	public void setTipoConsultaSEARCH() {
		this.tipo = TipoConsulta.SEARCH;
	}

	/**
	 * Sets the triples.
	 *
	 * @param triples
	 *            the new triples
	 */
	public void setTriples(List<Triple> triples) {
		this.triples = triples;
	}

}
