/*
 * @author Elena Hervalejo
 */
package org.hdtourist.model;

import java.io.File;
import java.io.Serializable;
import java.math.BigDecimal;

import org.hdtourist.model.services.FicherosService;
import org.hdtourist.utils.HdtException;

import android.util.Log;

/**
 * La clase que representa el fichero HDT y sus datos
 */
public class FicheroHdt implements Serializable {

	private static final String LOG_TAG = "FicheroHdt";

	/** La constante serialVersionUID. */
	private static final long serialVersionUID = 1L;

	/** The inicio. */
	private boolean inicio = true;

	/** The nombre. */
	private String nombre;

	/** El tamao del fichero HDT. */
	private BigDecimal tamano;

	/** El tamao del fichero de mapas comprimido. */
	private BigDecimal tamanoMaps;

	/** The uri. */
	private String uri;

	/** The uri original. */
	private String uriOriginal;

	/**
	 * Comprueba y fija uri.
	 *
	 * @param categoria
	 *            the categoria
	 */
	public void compruebaYFijaUri(Categoria categoria) {
		if (categoria.getUri() != null) {
			this.setUri(categoria.getUri());
		} else {
			this.setUri(this.uriOriginal);
		}
	}

	/**
	 * Gets the fichero.
	 *
	 * @return the fichero
	 */
	public File getFichero() {
		return new File(this.getRutaFichero());
	}

	/**
	 * Gets the mapa dir.
	 *
	 * @return the mapa dir
	 * @throws HdtException
	 *             Excepcin de la aplicacin
	 */
	public File getMapaDir() throws HdtException {
		return new File(this.getRutaMapa());
	}

	/**
	 * Gets the nombre.
	 *
	 * @return the nombre
	 */
	public String getNombre() {
		return this.nombre;
	}
	
	public String getNombreSinEspacios() {
		return this.nombre.replaceAll(" ", "_");
	}

	/**
	 * Gets the ruta fichero.
	 *
	 * @return the ruta fichero
	 */
	public String getRutaFichero() {
		try {
			FicherosService.getInstance();
			return FicherosService.obtenDirectorioHdts().getAbsolutePath() + File.separator + this.getNombreSinEspacios() + ".hdt";
		} catch (HdtException e) {
			return null;
		}
	}

	/**
	 * Gets the ruta mapa.
	 *
	 * @return the ruta mapa
	 * @throws HdtException
	 *             Excepcin de la aplicacin
	 */
	public String getRutaMapa() throws HdtException {
		try {
			return FicherosService.getInstance().obtenDirectorioMapas() + File.separator + this.getNombreSinEspacios();
		} catch (HdtException e) {
			throw e;
		}
	}

	/**
	 * Gets the tamano.
	 *
	 * @return the tamano
	 */
	public BigDecimal getTamano() {
		return this.tamano;
	}

	/**
	 * Gets the uri.
	 *
	 * @return the uri
	 */
	public String getUri() {
		return this.uri;
	}

	/**
	 * Hay mapas offline.
	 *
	 * @return true, if successful
	 */
	public boolean hayMapasOffline() {
		try {
			File mapasDir = this.getMapaDir();
			if (!mapasDir.exists()) {
				return false;
			}
			boolean hay = false;
			boolean hayConf = false;
			for (File f : mapasDir.listFiles()) {
				if (f.getName().indexOf("cache") >= 0) {
					hayConf = true;
				} else if (f.getName().indexOf(this.getNombreSinEspacios()) >= 0) {
					hay = true;
				}
			}
			return hay && hayConf;
		} catch (HdtException e) {
			Log.e(FicheroHdt.LOG_TAG, "Error al comprobar si hay mapas", e);
			return false;
		}
	}

	/**
	 * Sets the fichero.
	 *
	 * @param fichero
	 *            the new fichero
	 */
	public void setFichero(File fichero) {
	}

	/**
	 * Sets the nombre.
	 *
	 * @param nombre
	 *            the new nombre
	 */
	public void setNombre(String nombre) {
		this.nombre = nombre;
	}

	/**
	 * Sets the ruta fichero.
	 *
	 * @param rutaFichero
	 *            the new ruta fichero
	 */
	public void setRutaFichero(String rutaFichero) {
	}

	/**
	 * Sets the tamano.
	 *
	 * @param tamano
	 *            the new tamano
	 */
	public void setTamano(BigDecimal tamano) {
		this.tamano = tamano;
	}

	/**
	 * Sets the tamano.
	 *
	 * @param tamano
	 *            the new tamano
	 */
	public void setTamano(int tamano) {
		this.tamano = BigDecimal.valueOf(tamano);
	}

	/**
	 * Sets the uri.
	 *
	 * @param uri
	 *            the new uri
	 */
	public void setUri(String uri) {
		if (this.inicio) { // La primera vez hacemos una copia
			this.uriOriginal = uri;
		}
		this.inicio = false;
		this.uri = uri;
	}

	/**
	 * @return the tamanoMaps
	 */
	public BigDecimal getTamanoMaps() {
		return tamanoMaps;
	}

	/**
	 * @param tamanoMaps
	 * 				el nuevo tamao del fichero HDT
	 */
	public void setTamanoMaps(BigDecimal tamanoMaps) {
		this.tamanoMaps = tamanoMaps;
	}

	/**
	 * @param tamanoMaps 
	 * 				el nuevo tamao del fichero de mapas comprimido
	 */
	public void setTamanoMaps(int tamanoMaps) {
		this.tamanoMaps = BigDecimal.valueOf(tamanoMaps);
	}

}
