/*
 * @author Elena Hervalejo
 */
package org.hdtourist.model;

/**
 * La clase triple, almacena la terna sujeot-predicado-valor
 */
public class Triple implements Cloneable {

	/**
	 * The Enum TipoObjetoConsultado.
	 */
	public enum TipoObjetoConsultado {

		/** The objeto. */
		OBJETO,
		/** The predicado. */
		PREDICADO,
		/** The sujeto. */
		SUJETO,
		/** The todos. */
		TODOS;
	}

	/** El encadena anterior. */
	private TipoObjetoConsultado encadenaAnterior;

	/** El objeto. */
	private String objeto;

	/** El objeto consultado. */
	private TipoObjetoConsultado objetoConsultado;

	/** El origen objeto. */
	private String origenObjeto;

	/** El predicado. */
	private String predicado;

	/** El sujeto. */
	private String sujeto;

	/**
	 * Instancia un nuevo triple.
	 */
	public Triple() {
		this.sujeto = "";
		this.predicado = "";
		this.objeto = "";
	}

	/**
	 * Instancia un nuevo triple.
	 *
	 * @param sujeto
	 *            el sujeto
	 * @param predicado
	 *            el predicado
	 * @param objeto
	 *            el objeto
	 */
	public Triple(String sujeto, String predicado, String objeto) {
		super();
		this.sujeto = sujeto;
		this.predicado = predicado;
		this.objeto = objeto;
	}

	/**
	 * Copiar triple.
	 *
	 * @return the triple
	 */
	public Triple copiarTriple() {
		try {
			return (Triple) this.clone();
		} catch (CloneNotSupportedException e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * Obtiene el objeto.
	 *
	 * @return el objeto
	 */
	public String getObjeto() {
		return this.objeto;
	}

	/**
	 * Obtiene el origen objeto.
	 *
	 * @return el origen objeto
	 */
	public String getOrigenObjeto() {
		return this.origenObjeto;
	}

	/**
	 * Obtiene el predicado.
	 *
	 * @return el predicado
	 */
	public String getPredicado() {
		return this.predicado;
	}

	/**
	 * Obtiene el resultado.
	 *
	 * @param tipoConsulta
	 *            el tipo de consulta
	 * @return el resultado
	 */
	public String getResultado(TipoObjetoConsultado tipoConsulta) {
		String res = "";
		if (tipoConsulta == TipoObjetoConsultado.OBJETO) {
			res = this.getObjeto();
		} else if (tipoConsulta == TipoObjetoConsultado.SUJETO) {
			res = this.getSujeto();
		} else if (tipoConsulta == TipoObjetoConsultado.TODOS) {
			res = this.getSujeto() + "\n" + this.getPredicado() + "\n" + this.getObjeto();
		}
		return res;
	}

	/**
	 * Obtiene el sujeto.
	 *
	 * @return el sujeto
	 */
	public String getSujeto() {
		return this.sujeto;
	}

	/**
	 * Obtiene el tipo objeto consultado.
	 *
	 * @return el tipo objeto consultado
	 */
	public TipoObjetoConsultado getTipoObjetoConsultado() {
		return this.objetoConsultado;
	}

	/**
	 * Indica que el objeto se sustituye con el resultado de la consulta anterior
	 *
	 * @return true, si se sustituye el objeto por el resultado de la consulta anterior
	 */
	public boolean isEncadenaAnteriorObjeto() {
		return this.encadenaAnterior == TipoObjetoConsultado.OBJETO;
	}

	/**
	 * Indica que el sujeto se sustituye con el resultado de la consulta anterior
	 *
	 * @return true, si se sustituye el sujeto por el resultado de la consulta anterior
	 */
	public boolean isEncadenaAnteriorSujeto() {
		return this.encadenaAnterior == TipoObjetoConsultado.SUJETO;
	}

	/**
	 * Indica si la consulta se refiere al objeto
	 *
	 * @return true, si lo que se consulta es el objeto
	 */
	public boolean isTipoObjetoConsultadoObjeto() {
		return this.objetoConsultado == TipoObjetoConsultado.OBJETO;
	}

	/**
	 * Indica si la consulta se refiere al sujeto
	 *
	 * @return true, si lo que se consulta es el sujeto
	 */
	public boolean isTipoObjetoConsultadoSujeto() {
		return this.objetoConsultado == TipoObjetoConsultado.SUJETO;
	}

	/**
	 * Configura que el objeto se sustituye con el resultado de la consulta anterior
	 */
	public void setEncadenaAnteriorObjeto() {
		this.encadenaAnterior = TipoObjetoConsultado.OBJETO;
	}

	/**
	 * Configura que el sujeto se sustituye con el resultado de la consulta anterior
	 */
	public void setEncadenaAnteriorSujeto() {
		this.encadenaAnterior = TipoObjetoConsultado.SUJETO;
	}

	/**
	 * Fija el objeto.
	 *
	 * @param objeto
	 *            the new objeto
	 */
	public void setObjeto(String objeto) {
		this.objeto = objeto;
	}

	/**
	 * Fija el origen objeto.
	 *
	 * @param origenObjeto
	 *            el nuevo origen objeto
	 */
	public void setOrigenObjeto(String origenObjeto) {
		this.origenObjeto = origenObjeto;
	}

	/**
	 * El predicado.
	 *
	 * @param predicado
	 *            el nuevo predicado
	 */
	public void setPredicado(String predicado) {
		this.predicado = predicado;
	}

	/**
	 * Fija el sujeto.
	 *
	 * @param sujeto
	 *            el nuevo sujeto
	 */
	public void setSujeto(String sujeto) {
		this.sujeto = sujeto;
	}

	/**
	 * Fija el tipo objeto consultado objeto.
	 */
	public void setTipoObjetoConsultadoObjeto() {
		this.objetoConsultado = TipoObjetoConsultado.OBJETO;
	}

	/**
	 * Fija el tipo objeto consultado sujeto.
	 */
	public void setTipoObjetoConsultadoSujeto() {
		this.objetoConsultado = TipoObjetoConsultado.SUJETO;
	}

	/**
	 * Fija el tipo objeto consultado todos.
	 */
	public void setTipoObjetoConsultadoTodos() {
		this.objetoConsultado = TipoObjetoConsultado.TODOS;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#clone()
	 */
	@Override
	protected Object clone() throws CloneNotSupportedException {
		return super.clone();
	}
}
