/*
 * @author Elena Hervalejo
 */
package org.hdtourist.model.services;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

import org.hdtourist.model.CategoriaCiudad;
import org.hdtourist.model.CategoriaMapa;
import org.hdtourist.model.Consulta;
import org.hdtourist.model.ConsultaTuberia;
import org.hdtourist.model.FicheroHdt;
import org.hdtourist.model.GeoPosicion;
import org.hdtourist.model.ListaGeoPosiciones;
import org.hdtourist.model.Triple;
import org.hdtourist.model.services.xml.InformacionConfigParser;
import org.hdtourist.presenter.InformacionListPresenter;
import org.hdtourist.utils.HdtException;
import org.hdtourist.utils.StringUtils;
import org.hdtourist.view.inf.InformacionDato;
import org.hdtourist.view.inf.InformacionDatoSimple;
import org.hdtourist.view.inf.InformacionGrupo;
import org.hdtourist.view.inf.InformacionLista;
import org.hdtourist.view.inf.InformacionListaVinculos;
import org.hdtourist.view.inf.InformacionVinculo;
import org.hdtourist.view.utils.HdTIdioma;

/**
 * Servicio que proporciona utilidades realcionadas con la informacin HDT.
 */
public class InformacionService {

	/** La constante CONSULTA_CENTRO_CIUDAD. */
	final private static String CONSULTA_CENTRO_CIUDAD = "conf_obtener_centro_ciudad";

	/** La constante CONSULTA_COORDENADAS_PUNTO. */
	final private static String CONSULTA_COORDENADAS_PUNTO = "conf_obtener_posicion";

	/** La constante CONSULTA_IMAGEN_CIUDAD. */
	final private static String CONSULTA_IMAGEN_CIUDAD = "conf_obtener_imagen_ciudad";

	/** La constante CONSULTA_NOMBRE_PUNTO. */
	final private static String CONSULTA_NOMBRE_PUNTO = "conf_obtener_nombre_punto";

	/** La constante CONSULTA_URI_PUNTOS. */
	final private static String CONSULTA_URI_PUNTOS = "conf_obtener_uri_puntos";

	/** The instance. */
	private static InformacionService INSTANCE = null;

	/** La constante MAX_RESULTS. */
	final private static int MAX_RESULTS = 100;

	/** El manejador. */
	ManejaHdt manejador = null;

	/** El ltimo triple (valor) de consulta del resultado (clave). */
	HashMap<String, Triple> resultUltTriple = new HashMap<String, Triple>();

	/**
	 * Instancia un nuevo objeto InformacionService
	 */
	private InformacionService() {
	}

	/**
	 * Obtiene la unica instanacia de InformacionService.
	 *
	 * @return single instance of InformacionService
	 */
	public static InformacionService getInstance() {
		InformacionService.createInstance();
		return InformacionService.INSTANCE;
	}

	/**
	 * Obtiene las consultas correspondientes del fichero de configuracin de los mapas: conf_mapa.xml
	 *
	 * @param etiquetaConsulta
	 *            CONSULTA_URI_PUNTOS: busca las uris de todos los puntos CONSULTA_NOMBRE_PUNTO: buscar el nombre dada una uri de las anteriores CONSULTA_COORDENADAS_PUNTO: busca la posicin del punto dada una uri de las anteriores CONSULTA_CENTRO_CIUDAD: obtiene el centro
	 *            geolocalizado de la ciudad, dada la uri de la ciudad
	 * @return the consulta
	 * @throws HdtException
	 *             Excepcin de la aplicacin
	 */
	public static Consulta obtenerConsultaMapa(String etiquetaConsulta) throws HdtException {
		CategoriaMapa categoriaMapa = new CategoriaMapa();
		InformacionListPresenter informacion = InformacionConfigParser.getInstance().obtenerInformacionConfig(categoriaMapa);
		for (InformacionGrupo g : informacion.getListaInformacion()) {
			for (InformacionDato d : g.getListaDatos()) {
				if (etiquetaConsulta.equals(d.getEtiqueta())) {
					return d.getConsulta();
				}
			}
		}
		return null;
	}

	/**
	 * Crea la instancia.
	 */
	private synchronized static void createInstance() {
		if (InformacionService.INSTANCE == null) {
			InformacionService.INSTANCE = new InformacionService();
		}
	}

	/**
	 * Corrige el signo las cordenadas segn a la ciudad a la que pertenecen.
	 *
	 * @param ciudad
	 *            the ciudad
	 * @param geo
	 *            the geo
	 * @return the geo posicion
	 * @throws HdtException
	 *             Excepcin de la aplicacin
	 */
	public GeoPosicion corregirPunto(FicheroHdt ciudad, GeoPosicion geo) throws HdtException {
		GeoPosicion geoCiudad = this.obtenerCentro(ciudad);
		float latitude = geo.latitude;
		float longitude = geo.longitude;
		if (geoCiudad != null) {
			if (geoCiudad.latitude * latitude < 0) {
				latitude = -latitude;
			}

			if (geoCiudad.longitude * longitude < 0) {
				longitude = -longitude;
			}
		}
		return new GeoPosicion(latitude, longitude);
	}

	/**
	 * Formatea el triple sustituyendo las palabras clave por su valor - ${URI} : por la uri del fichero actual (uri de la ciudad o del vinculo) - ${NOMBRE} : por el nombre pasado en el fichero (nombre de la ciudad o del vinculo).
	 *
	 * @param triple
	 *            the triple
	 * @param fichero
	 *            the fichero
	 */
	public void formatoTriple(Triple triple, FicheroHdt fichero) {
		this.formatoTriple(triple, fichero, null, null);
	}

	/**
	 * Formatea el triple sustituyendo las palabras clave por su valor - ${URI} : por la uri del fichero actual (uri de la ciudad o del vinculo) - ${NOMBRE} : por el nombre pasado en el fichero (nombre de la ciudad o del vinculo).
	 *
	 * @param triple
	 *            el triple a formatear
	 * @param fichero
	 *            el fichero HDT sobre el que se realizar la consulta
	 * @param texto
	 *            el criterio de bsqueda si viene de una bsqueda
	 * @param uri
	 *            la uri
	 */
	public void formatoTriple(Triple triple, FicheroHdt fichero, String texto, String uri) {
		String uriSust = uri;
		if (StringUtils.isEmpty(uri)) {
			uriSust = fichero.getUri();
		}
		if ("${URI}".equals(triple.getSujeto())) {
			triple.setSujeto(uriSust);
		}
		if ("${URI}".equals(triple.getObjeto())) {
			triple.setObjeto(uriSust);
		}
		if ("${URI}".equals(triple.getPredicado())) {
			triple.setPredicado(uriSust);
		}

		triple.setSujeto(triple.getSujeto().replace("${NOMBRE}", fichero.getNombre()));

		triple.setObjeto(triple.getObjeto().replace("${NOMBRE}", fichero.getNombre()));

		triple.setPredicado(triple.getPredicado().replace("${NOMBRE}", fichero.getNombre()));
	}

	/**
	 * Obtiene el sujeto triple de resultado.
	 *
	 * @param resultado
	 *            el resultado
	 * @return el sujeto triple de resultado
	 */
	public String getSujetoTripleDeResultado(String resultado) {
		Triple t = this.resultUltTriple.get(resultado);
		return t.getSujeto();
	}

	/**
	 * Informa el dato pasado por parmetro realizando las consultas que contiene.
	 *
	 * @param dato
	 *            el dato
	 * @param fichero
	 *            el fichero
	 * @param uri
	 *            la uri
	 */
	public void informaDato(InformacionDato dato, FicheroHdt fichero, String uri) {
		Consulta consulta = dato.getConsulta();
		HashSet<String> listado = this.resuelveConsulta(consulta, fichero, uri);
		if (dato instanceof InformacionListaVinculos) {
			List<InformacionVinculo> listadoDatosC = new ArrayList<InformacionVinculo>();
			for (String elemento : listado) {
				InformacionVinculo vinculo = new InformacionVinculo();
				vinculo.setNombre(elemento);
				vinculo.setFicheroConf(dato.getCategoria().getFicheroConf());
				vinculo.setUri(this.getSujetoTripleDeResultado(elemento));
				listadoDatosC.add(vinculo);
			}
			((InformacionListaVinculos) dato).setVinculos(listadoDatosC);
		} else if (dato instanceof InformacionLista) {
			List<InformacionDatoSimple> listadoDatosC = new ArrayList<InformacionDatoSimple>();
			for (String elemento : listado) {
				InformacionDatoSimple dato2 = new InformacionDatoSimple();
				dato2.setValores(new HashSet<String>(Arrays.asList(elemento)));
				listadoDatosC.add(dato2);
			}
			((InformacionLista) dato).setDatos(listadoDatosC);
		}
		dato.setValores(listado);
	}

	/**
	 * Informa los datos pasados dentro del parametro Informacion.
	 *
	 * @param informacion
	 *            el informacion
	 * @param fichero
	 *            el fichero HDT
	 * @param uri
	 *            la uri
	 */
	public void informaFicheroHdt(InformacionListPresenter informacion, FicheroHdt fichero, String uri) {
		this.getManejador(fichero);
		for (InformacionGrupo grupo : informacion.getListaInformacion()) {
			for (InformacionDato dato : grupo.getListaDatos()) {
				this.informaDato(dato, fichero, uri);
			}
		}
	}

	/**
	 * Obtiene el centro de la ciudad, segn la consulta correspondiente del fichero conf_mapa.
	 *
	 * @param ciudad
	 *            la ciudad
	 * @return the geo posicion
	 * @throws HdtException
	 *             Excepcin de la aplicacin
	 */
	public GeoPosicion obtenerCentro(FicheroHdt ciudad) throws HdtException {
		this.getManejador(ciudad);
		Consulta consultaCentro = InformacionService.obtenerConsultaMapa(InformacionService.CONSULTA_CENTRO_CIUDAD);
		Iterator<String> listaCentro = this.resuelveConsulta(consultaCentro, ciudad, ciudad.getUri()).iterator();
		if (listaCentro.hasNext()) {
			return GeoPosicion.getGeoPosicion(listaCentro.next());
		}
		return null;
	}

	/**
	 * Informa la uri para los vnculos.
	 *
	 * @param elemento
	 *            el elemento
	 * @param consulta
	 *            el consulta
	 * @param fichero
	 *            el fichero
	 * @return la uri
	 */
	public String obtenerUri(String elemento, Consulta consulta, FicheroHdt fichero) {
		String aux = fichero.getNombre();
		fichero.setNombre(elemento);
		Iterator<String> i = this.resuelveConsulta(consulta, fichero, null).iterator();
		String uri = "";
		if (i.hasNext()) {
			uri = i.next();
		} else {
			fichero.setNombre("\"" + elemento + "\"@" + HdTIdioma.getLocaleString());
			i = this.resuelveConsulta(consulta, fichero, null).iterator();
			if (i.hasNext()) {
				uri = i.next();
			} else {
				fichero.setNombre("\"" + elemento + "\"@" + HdTIdioma.getAlternateLocaleString());
				i = this.resuelveConsulta(consulta, fichero, null).iterator();
				if (i.hasNext()) {
					uri = i.next();
				}
			}
		}
		fichero.setNombre(aux);
		return uri;
	}

	/**
	 * Obtiene la url de la imagen de la ciudad.
	 *
	 * @param ciudad
	 *            Fichero de la ciudad para la que consultamos la imagen
	 * @return the string
	 * @throws HdtException
	 *             Excepcin de la aplicacin
	 */
	public String obtenerUrlImagenCiudad(FicheroHdt ciudad) throws HdtException {
		CategoriaCiudad categoriaMapa = new CategoriaCiudad();
		InformacionListPresenter informacion = InformacionConfigParser.getInstance().obtenerInformacionConfig(categoriaMapa);
		for (InformacionGrupo g : informacion.getListaInformacion()) {
			for (InformacionDato d : g.getListaDatos()) {
				if (InformacionService.CONSULTA_IMAGEN_CIUDAD.equals(d.getEtiqueta())) {
					Consulta consulta = d.getConsulta();
					this.getManejador(ciudad);
					Iterator<String> listaImagen = this.resuelveConsulta(consulta, ciudad, ciudad.getUri()).iterator();
					if (listaImagen.hasNext()) {
						return listaImagen.next();
					}
				}
			}
		}
		return null;
	}

	/**
	 * Obten puntos del mapa.
	 *
	 * @param fichero
	 *            el fichero
	 * @return el lista geo posiciones
	 * @throws HdtException
	 *             Excepcin de la aplicacin
	 */
	public ListaGeoPosiciones obtenPuntosDelMapa(FicheroHdt fichero) throws HdtException {
		this.getManejador(fichero);
		ListaGeoPosiciones lista = new ListaGeoPosiciones();
		lista.setFichero(fichero);
		Consulta consultaUriTodos = InformacionService.obtenerConsultaMapa(InformacionService.CONSULTA_URI_PUNTOS);

		HashSet<String> listaUris = this.resuelveConsulta(consultaUriTodos, fichero, fichero.getUri());

		for (String uri : listaUris) {
			Consulta consultaNombrePunto = InformacionService.obtenerConsultaMapa(InformacionService.CONSULTA_NOMBRE_PUNTO);
			Iterator<String> listaNombre = this.resuelveConsulta(consultaNombrePunto, fichero, uri).iterator();
			if (listaNombre.hasNext()) {
				Consulta consultaPunto = InformacionService.obtenerConsultaMapa(InformacionService.CONSULTA_COORDENADAS_PUNTO);
				Iterator<String> listaPunto = this.resuelveConsulta(consultaPunto, fichero, uri).iterator();
				if (listaPunto.hasNext()) {
					lista.add(this.corregirPunto(fichero, GeoPosicion.getGeoPosicion(listaPunto.next())), listaNombre.next(), uri, fichero);
				}
			}
		}
		GeoPosicion centro = this.obtenerCentro(fichero);
		if (centro != null) {
			lista.setCentro(centro);
		}
		return lista;

	}

	/**
	 * Resuelve la consulta pasada por parmetro.
	 *
	 * @param consulta
	 *            la consulta
	 * @param fichero
	 *            el fichero HDT sobre el que se hace las consultas
	 * @param uri
	 *            la uri
	 * @return Lista de resultados
	 */
	public HashSet<String> resuelveConsulta(Consulta consulta, FicheroHdt fichero, String uri) {
		this.resultUltTriple = new HashMap<String, Triple>();
		HashSet<String> resultados = new HashSet<String>();
		boolean primera = true;
		for (ConsultaTuberia consultaT : consulta.getConsultasTuberia()) {
			List<String> res = this.resuelveConsultaTuberia(consultaT, fichero, consulta.getTexto(), uri);
			if (consulta.isTipoConsultaOR() || primera) {
				resultados.addAll(res);
			} else if (consulta.isTipoConsultaAND()) {
				resultados.retainAll(res);
			}
			primera = false;
		}
		for (Triple triple : consulta.getTriples()) {
			List<String> res = this.resuelveConsultaSencilla(triple, fichero, consulta.getTexto(), uri);
			if (consulta.isTipoConsultaOR() || primera) {
				resultados.addAll(res);
			} else if (consulta.isTipoConsultaAND()) {
				resultados.retainAll(res);
			}
			primera = false;
		}
		return resultados;
	}

	/**
	 * Resuelve consulta sencilla.
	 *
	 * @param triple
	 *            el triple de la consulta
	 * @param fichero
	 *            el fichero HDT
	 * @param texto
	 *            el criterio de bsqueda
	 * @param uri
	 *            la uri
	 * @return la lista de resultados
	 */
	public List<String> resuelveConsultaSencilla(Triple triple, FicheroHdt fichero, String texto, String uri) {
		List<String> resultados = new ArrayList<String>();
		triple = triple.copiarTriple();
		this.formatoTriple(triple, fichero, texto, uri);
		List<Triple> resultado = new ArrayList<Triple>();
		if (StringUtils.isEmpty(texto)) {
			resultado = this.manejador.consulta(triple, HdTIdioma.getLocaleString());
		} else {
			resultado = this.manejador.consultaBusqueda(triple, HdTIdioma.getLocaleString(), texto);
		}
		int numRes = 0;
		for (Triple tr : resultado) {
			String res = "";
			res = tr.getResultado(triple.getTipoObjetoConsultado());
			resultados.add(res);
			this.resultUltTriple.put(res, triple);
			numRes++;
			if (numRes >= InformacionService.MAX_RESULTS) {
				break;
			}
		}
		return resultados;
	}

	/**
	 * Resuelve consulta tuberia.
	 *
	 * @param consulta
	 *            la consulta
	 * @param fichero
	 *            el fichero HDT sobre el que se realiza la consulta
	 * @param texto
	 *            el criterio de bsqueda
	 * @param uri
	 *            la uri
	 * @return la lista de resultados
	 */
	public List<String> resuelveConsultaTuberia(ConsultaTuberia consulta, FicheroHdt fichero, String texto, String uri) {
		List<String> resultados = this.resuelveTuberiaRecursivo(consulta.getTriples(), "", fichero, texto, uri);
		return resultados;
	}

	/**
	 * Resuelve tuberia recursivo.
	 *
	 * @param triples
	 *            the triples
	 * @param resActual
	 *            the res actual
	 * @param fichero
	 *            el fichero HDT sobre el que se realiza la consulta
	 * @param texto
	 *            el criterio de bsqueda
	 * @param uri
	 *            la uri
	 * @return la lista de resultados
	 */
	public List<String> resuelveTuberiaRecursivo(List<Triple> triples, String resActual, FicheroHdt fichero, String texto, String uri) {
		Triple triple = triples.get(0);
		if (StringUtils.isNotEmpty(resActual)) {
			if (triple.isEncadenaAnteriorObjeto()) {
				triple.setObjeto(resActual);
			} else if (triple.isEncadenaAnteriorSujeto()) {
				triple.setSujeto(resActual);
			}
		}
		triple = triple.copiarTriple();
		this.formatoTriple(triple, fichero, null, uri);
		List<Triple> resultado = null;
		if (StringUtils.isEmpty(texto)) {
			resultado = this.manejador.consulta(triple, HdTIdioma.getLocaleString());
		} else {
			resultado = this.manejador.consultaBusqueda(triple, HdTIdioma.getLocaleString(), texto);
		}
		ArrayList<String> resultados = new ArrayList<String>();
		for (Triple tr : resultado) {
			resActual = tr.getResultado(triple.getTipoObjetoConsultado());
			if (triples.size() == 1) {
				this.resultUltTriple.put(resActual, triple);
				resultados.add(resActual);
			} else {
				List<Triple> aux = triples.subList(1, triples.size());
				resultados.addAll(this.resuelveTuberiaRecursivo(aux, resActual, fichero, texto, uri));
			}
		}
		return resultados;
	}

	/**
	 * Gets the manejador.
	 *
	 * @param fichero
	 *            the fichero
	 * @return the manejador
	 */
	private ManejaHdt getManejador(FicheroHdt fichero) {
		if (this.manejador == null) {
			this.manejador = ManejaHdt.getInstance();
		}
		this.manejador.cargaHdt(fichero.getFichero());
		return this.manejador;
	}
}
