/*
 * @author Elena Hervalejo
 */
package org.hdtourist.model.services;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.math.BigDecimal;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.hdtourist.R;
import org.hdtourist.model.Triple;
import org.hdtourist.utils.StringUtils;
import org.hdtourist.view.utils.HdTIdioma;
import org.rdfhdt.hdt.exceptions.NotFoundException;
import org.rdfhdt.hdt.hdt.HDT;
import org.rdfhdt.hdt.hdt.HDTManager;
import org.rdfhdt.hdt.triples.IteratorTripleString;
import org.rdfhdt.hdt.triples.TripleString;

import android.util.Log;

/**
 * Manejador de ficheros HDT.
 */
public class ManejaHdt {

	/** La instancia. */
	private static ManejaHdt INSTANCE = null;

	/** La constante LOG_TAG. */
	private final static String LOG_TAG = "ManejaHdt";

	/** La constante MAX_RESULTS. */
	final private static int MAX_RESULTS = 100;

	/** The hdt. */
	private HDT hdt;

	/**
	 * Instancia un nuevo maneja hdt.
	 */
	private ManejaHdt() {
	}

	/**
	 * Obtiene la instancia unica de ManejaHdt.
	 *
	 * @return obtiene la instancia ManejaHdt
	 */
	public static ManejaHdt getInstance() {
		ManejaHdt.createInstance();
		return ManejaHdt.INSTANCE;
	}

	/**
	 * Crea la instancia.
	 */
	private synchronized static void createInstance() {
		if (ManejaHdt.INSTANCE == null) {
			ManejaHdt.INSTANCE = new ManejaHdt();
		}
	}

	/**
	 * Carga hdt a partir de un archivo en formato HDT.
	 *
	 * @param f
	 *            archivo HDT
	 */
	public void cargaHdt(File f) {

		BufferedInputStream fraw;
		try {
			fraw = new BufferedInputStream(new FileInputStream(f.getAbsolutePath()));
			this.hdt = HDTManager.loadHDT(fraw, null);
		} catch (FileNotFoundException e) {
			Log.e(ManejaHdt.LOG_TAG + " cargaHDT", "Archivo no encontrado", e);
			e.printStackTrace();
		} catch (IOException e) {
			Log.e(ManejaHdt.LOG_TAG + " cargaHDT", "Error de entrada/salilda", e);
			e.printStackTrace();
		}
	}
	
	/**
	 * Obtiene el objeto HDT al que realizar las consultas
	 *
	 * @return el objeto HDT
	 */
	public HDT getHDT() {
		return this.hdt;
	}

	/**
	 * Realiza la consulta dada por el triple (sujeto, predicado, objeto)
	 *
	 * @param triple
	 *            el triple
	 * @return la lista de triples que encajan con el valor (sujeot, predicado, objeto)
	 */
	public List<Triple> consulta(Triple triple) {
		return this.consulta(triple, null);
	}

	/**
	 * Realiza la consulta dada por el triple (sujeto, predicado, objeto)
	 * Obtiene los resultados de acuerdo con el idioma pasado por parmetro.
	 *
	 * @param triple
	 *            el triple
	 * @param language
	 *            el idioma, si el idioma es null, devuelve todos los resultados
	 * @return la lista de triples que encajan con el valor (sujeot, predicado, objeto) y el idioma pasado
	 */
	public List<Triple> consulta(Triple triple, String language) {
		return this.consulta(triple, language, true);
	}

	/**
	 * Realiza la consulta dada por el triple (sujeto, predicado, objeto)
	 * Obtiene los resultados de acuerdo con el idioma pasado por parmetro.
	 *
	 * @param triple
	 *            el triple
	 * @param language
	 *            el idioma, si el idioma es null, devuelve todos los resultados
	 * @param maximo
	 *            true si se aplica el maximo de resultados: MX_RESULTS
	 * @return la lista de triples que encajan con el valor (sujeot, predicado, objeto) y el idioma pasado
	 */
	public List<Triple> consulta(Triple triple, String language, boolean maximo) {

		List<Triple> result = new ArrayList<Triple>();

		try {
			IteratorTripleString it = null;
			it = this.hdt.search(triple.getSujeto(), triple.getPredicado(), triple.getObjeto());
			while (it.hasNext()) {
				TripleString ts = it.next();
				result.addAll(this.fixResult(language, ts));
				Log.v(ManejaHdt.LOG_TAG + " consulta", ts.toString() + ts.getObject().toString());
				if (maximo && result.size() >= ManejaHdt.MAX_RESULTS) {
					break;
				}
			}
			// Si no ha encontrado ninguno en el idioma, pero si hay en otros, cojemos otros idiomas
			if (result.size() == 0 && it.estimatedNumResults() > 0) {
				it.goToStart();
				while (it.hasNext()) {
					TripleString ts = it.next();
					result.addAll(this.fixResult(HdTIdioma.getAlternateLocaleString(), ts));
				}
			}
			if (result.size() == 0 && it.estimatedNumResults() > 0) {
				it.goToStart();
				while (it.hasNext()) {
					TripleString ts = it.next();
					result.addAll(this.fixResult(null, ts));
					if (maximo && result.size() >= ManejaHdt.MAX_RESULTS) {
						break;
					}
				}
			}
		} catch (NotFoundException e) {
			Log.i(this.getClass().getName(), "No encontrado", e);
			e.printStackTrace();
		}

		return result;
	}

	/**
	 * Realiza la consulta sin buscar un objeto, predicado o sujeto concreto, si no buscando en el texto del objeto, predicado o sujeto, dependiendo de cual de los tres tenga la clave ${TEXTO}.
	 *
	 * @param triple
	 *            el triple
	 * @param language
	 *            el idioma, si el idioma es null, devuelve todos los resultados
	 * @param texto
	 *            el texto buscado
	 * @return la lista de triples que encajan con el valor (sujeot, predicado, objeto) y el idioma pasado
	 */
	public List<Triple> consultaBusqueda(Triple triple, String language, String texto) {
		List<Triple> resultados = new ArrayList<Triple>();
		boolean textoEnSujeto = triple.getSujeto().indexOf("${TEXTO}") >= 0;
		boolean textoEnPredicado = triple.getPredicado().indexOf("${TEXTO}") >= 0;
		boolean textoEnObjeto = triple.getObjeto().indexOf("${TEXTO}") >= 0;
		triple.setSujeto(triple.getSujeto().replace("${TEXTO}", ""));
		triple.setPredicado(triple.getPredicado().replace("${TEXTO}", ""));
		triple.setObjeto(triple.getObjeto().replace("${TEXTO}", ""));

		List<Triple> result = this.consulta(triple, language, false);
		for (Triple ts : result) {
			boolean encaja = true;
			if (textoEnObjeto && StringUtils.isNotEmpty(ts.getObjeto())) {
				encaja = encaja && ts.getObjeto().toString().toLowerCase().indexOf(texto.toLowerCase()) >= 0;
			}
			if (textoEnPredicado && StringUtils.isNotEmpty(ts.getPredicado())) {
				encaja = encaja && ts.getPredicado().toString().toLowerCase().indexOf(texto.toLowerCase()) >= 0;
			}
			if (textoEnSujeto && StringUtils.isNotEmpty(ts.getSujeto())) {
				encaja = encaja && ts.getSujeto().toString().toLowerCase().indexOf(texto.toLowerCase()) >= 0;
			}
			if (encaja) {
				resultados.addAll(this.fixResult(language, ts));
			}
			Log.v(ManejaHdt.LOG_TAG + " consulta manual", ts.toString() + ts.getObjeto().toString());
		}

		return resultados;
	}

	/**
	 * Obtener la uri de la ciudad
	 *
	 * @return la uri de la ciduad
	 */
	public String getCiudadUri() {
		return this.hdt.getBaseURI();
	}

	/**
	 * Dado un triple, devuelve el objeto en una lista si este est en el idioma indicado. Si no se indica idioma, lo devuevle tal cual. Se formatea el objeto tambin (se quitan las comillas, y la indicacin del tipo)
	 *
	 * @param language
	 *            el idioma, si el idioma es null, devuelve todos los resultados
	 * @param ts
	 *            el triple
	 * @return la lista de triples que encajan con el valor (sujeot, predicado, objeto) y el idioma pasado
	 */
	private List<Triple> fixResult(String language, Triple ts) {
		int i = ts.getObjeto().toString().indexOf("@");
		if (language == null || i < 0 || i >= 0 && ts.getObjeto().toString().substring(i + 1).equals(language)) {
			String object = ts.getObjeto().toString();
			if (i >= 0) {
				object = object.substring(0, i).replaceAll("\"", "");
			}
			object = this.formato(object);
			Triple resultado = new Triple(ts.getSujeto().toString(), ts.getPredicado().toString(), object);
			resultado.setOrigenObjeto(ts.getObjeto());
			return Arrays.asList(resultado);
		}
		return new ArrayList<Triple>();
	}

	/**
	 * Fix result.
	 *
	 * @param language
	 *            el idioma, si el idioma es null, devuelve todos los resultados
	 * @param ts
	 *            el triple
	 * @return la lista de triples que encajan con el valor (sujeot, predicado, objeto) y el idioma pasado
	 */
	private List<Triple> fixResult(String language, TripleString ts) {
		int i = ts.getObject().toString().indexOf("@");
		if (language == null || i < 0 || i >= 0 && ts.getObject().toString().substring(i + 1).equals(language)) {
			String object = ts.getObject().toString();
			if (i >= 0) {
				object = object.substring(0, i).replaceAll("\"", "");
			}
			object = this.formato(object);
			Triple resultado = new Triple(ts.getSubject().toString(), ts.getPredicate().toString(), object);
			resultado.setOrigenObjeto(ts.getObject().toString());
			return Arrays.asList(resultado);
		}
		return new ArrayList<Triple>();
	}

	/**
	 * Formatea el objeto.
	 *
	 * @param object
	 *            el objecto
	 * @return la cadena resultado formateado
	 */
	private String formato(String object) {
		// TODO Auto-generated method stub
		int i = object.indexOf("^^");
		if (i > 0) {
			if (object.indexOf("float", i) >= 0 || object.indexOf("double", i) >= 0 || object.indexOf("int", i) >= 0) {
				NumberFormat nf = NumberFormat.getInstance(HdTIdioma.getLocale());
				object = object.substring(0, i).replaceAll("\"", "");
				BigDecimal b = new BigDecimal(object);
				object = nf.format(b.doubleValue());
			} else if (object.indexOf("inhabitantsPerSquareKilometre", i) >= 0) {
				NumberFormat nf = NumberFormat.getInstance(HdTIdioma.getLocale());
				object = object.substring(0, i).replaceAll("\"", "");
				BigDecimal b = new BigDecimal(object);
				object = nf.format(b.doubleValue()) + " " + HdTIdioma.getString(R.string.informacion_tipo_personasKm2);
			} else if (object.indexOf("squareKilometre", i) >= 0) {
				NumberFormat nf = NumberFormat.getInstance(HdTIdioma.getLocale());
				object = object.substring(0, i).replaceAll("\"", "");
				BigDecimal b = new BigDecimal(object);
				object = nf.format(b.doubleValue()) + " " + HdTIdioma.getString(R.string.informacion_tipo_km2);
			} else if (object.indexOf("Geometry", i) >= 0 && object.indexOf("POINT") == 0) {// "POINT(-3.70023 40.4178)"^^http://www.openlinksw.com/schemas/virtrdf#Geometry
				object = object.substring(0, i).replaceAll("\"", "");
			} else {
				object = object.substring(0, i).replaceAll("\"", "");
			}
		}
		return object;
	}
}
