/*
 * @author Elena Hervalejo
 */
package org.hdtourist.model.services;

import android.content.SharedPreferences;
import android.preference.PreferenceManager;

/**
 * PreferencesService.
 * Clase de utilera para obtener la configuracion de la aplicacin.
 */
public class PreferencesService {

	/**
	 * Enumeracin del tipo de Conexion.
	 */
	public enum Conexion {

		/** The no. */
		NO("NO"),
		/** The preguntar. */
		PREGUNTAR("PREGUNTAR"),
		/** The si. */
		SI("SI");

		/** La conexion. */
		String conexion;

		/**
		 * Instancia el tipo de conexin
		 *
		 * @param con
		 *            el tipo de conexin
		 */
		Conexion(String con) {
			this.conexion = con;
		}

		/**
		 * Obtiene el tipo de conexin.
		 *
		 * @param conexion
		 *            the conexion
		 * @return the conexion
		 */
		public static Conexion get(String conexion) {
			if ("NO".equals(conexion)) {
				return NO;
			} else if ("SI".equals(conexion)) {
				return SI;
			} else {
				return PREGUNTAR;
			}
		}

		/*
		 * (non-Javadoc)
		 *
		 * @see java.lang.Enum#toString()
		 */
		@Override
		public String toString() {
			return this.conexion;
		}
	}

	/** La constante PREF_CONEXION. */
	public static final String PREF_CONEXION = "pref_conexion";

	/** La constante PREF_CONF_DIRECTORY. */
	public static final String PREF_CONF_DIRECTORY = "pref_directorio_configuraciones";

	/** La constante PREF_DESCARGAR_IMAGENES. */
	public static final String PREF_DESCARGAR_IMAGENES = "pref_descargar_imagenes";

	/** La constante PREF_MAPAS_3G. */
	public static final String PREF_MAPAS_3G = "pref_mapas_3g";

	/** La constante PREF_MAPAS_GOOGLE. */
	public static final String PREF_MAPAS_GOOGLE = "pref_mapas_google";

	/** La constante PREF_SERVIDOR. */
	public static final String PREF_SERVIDOR = "pref_servidor";

	/**
	 * Obtiene el valor booleano de la propiedad key configurada
	 *
	 * @param key
	 *            la clave de la propiedad
	 * @return el valor booleano de la propiedad
	 */
	public static boolean getBooleanPreference(String key) {
		SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(HdTouristService.getInstance().getContexto());
		return pref.getBoolean(key, true);
	}

	/**
	 * Obtiene el tipo de conexin configurado
	 *
	 * @return la conexin:
	 * 			- NO: No est permitida la conexin a Internet
	 * 			- PREGUNTAR: Se debe preguntar al usuario antes de conectarse a Internet
	 * 			- SI: Conectarse siempre sin preguntar
	 */
	public static Conexion getConexion() {
		SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(HdTouristService.getInstance().getContexto());
		return Conexion.get(pref.getString(PreferencesService.PREF_CONEXION, Conexion.PREGUNTAR.toString()));
	}

	/**
	 * Obtiene la propiedad booleana descargar imagenes.
	 * @return
	 *			- true, la descarga de imgenes est permitida
	 *			- false, la descarga de imgenes no est permitida
	 */
	public static boolean getDescargarImagenes() {
		SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(HdTouristService.getInstance().getContexto());
		return pref.getBoolean(PreferencesService.PREF_DESCARGAR_IMAGENES, false);
	}

	/**
	 * Obtiene la propiedad que indica si los mapas se pueden descargar por 3g
	 *
	 * @return la descargar mapas 3g
	 * 			- true: la descarga de mapas a travs de la red de datos mvil est permitida
	 * 			- false: la descarga de mapas a travs de la red de datos mvil no est permitida
	 */
	public static boolean getDescargarMapas3G() {
		SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(HdTouristService.getInstance().getContexto());
		return pref.getBoolean(PreferencesService.PREF_MAPAS_3G, false);
	}

	/**
	 * Obtiene el directorio configurado para almacenar los archivos de la aplicacin
	 *
	 * @return el directorio
	 */
	public static String getDirectorio() {
		SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(HdTouristService.getInstance().getContexto());
		return pref.getString(PreferencesService.PREF_CONF_DIRECTORY, FicherosService.obtenDirectorioPorDefecto());
	}

	/**
	 * Obtiene la url de los hdt en el servidor
	 *
	 * @return the hdts url
	 */
	public static String getHdtsUrl() {
		return PreferencesService.getServidor() + "hdts/";
	}

	/**
	 * Obtiene la url de los mapas en el servidor
	 *
	 * @return the maps url
	 */
	public static String getMapsUrl() {
		return PreferencesService.getServidor() + "mapas/";
	}

	/**
	 * Obtiene la url del servidor
	 *
	 * @return the servidor
	 */
	public static String getServidor() {
		SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(HdTouristService.getInstance().getContexto());
		return pref.getString(PreferencesService.PREF_SERVIDOR, "http://alumnos.inf.uva.es/eleherv/");
	}

	/**
	 * Obtiene el valor de la propiedad key configurada
	 *
	 * @param key
	 *            the key
	 * @return the string preference
	 */
	public static String getStringPreference(String key) {
		SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(HdTouristService.getInstance().getContexto());
		return pref.getString(key, "");
	}

	/**
	 * Obtiene el valor configurado para el uso o no de los mapas de google
	 *
	 * @return usar mapas google
	 * 		- true si el uso de los mapas de google est activado
	 * 		- false si el suo de los mapas de google est desactivado
	 */
	public static boolean getUsarMapasGoogle() {
		SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(HdTouristService.getInstance().getContexto());
		return pref.getBoolean(PreferencesService.PREF_MAPAS_GOOGLE, false);
	}

	/**
	 * @return  
	 * 			- True si la conexin est establecida a no conectarse nunca
	 * 			- False en caso contrario
	 */
	public static boolean isConexionNunca() {
		return Conexion.NO == PreferencesService.getConexion();
	}

	/**
	 * @return 
	 * 			- True, si la conexin est establecida a preguntar antes de conectar al usuario
	 * 			- False, en caso contrario
	 */
	public static boolean isConexionPreguntar() {
		return Conexion.PREGUNTAR == PreferencesService.getConexion();
	}

	/**
	 * @return 
	 * 			- True, si la conexin est establecida a conectarse sin preguntar siempre
	 * 			- False en caso contrario
	 */
	public static boolean isConexionSiempre() {
		return Conexion.SI == PreferencesService.getConexion();
	}
}
