/*
 * @author Elena Hervalejo
 */
package org.hdtourist.presenter;

import java.io.File;

import org.hdtourist.R;
import org.hdtourist.model.FicheroHdt;
import org.hdtourist.model.services.FicherosService;
import org.hdtourist.model.services.PreferencesService;
import org.hdtourist.utils.HdtException;
import org.hdtourist.view.CiudadActivity;
import org.hdtourist.view.base.BaseActionBarActivity;
import org.hdtourist.view.base.BaseFragment;
import org.hdtourist.view.utils.BarProgressDialog;
import org.hdtourist.view.utils.DialogAlerta;
import org.hdtourist.view.utils.HdTIdioma;
import org.hdtourist.view.utils.IProgress;

import android.content.Intent;

/**
 * Presentador de la arquitectura M-V-P Para la actividades que contienen una lista de ciudades hdt que puedan descargarse.
 *
 * @author Elena Hervalejo
 */
public abstract class CiudadesListaPresenter {

	/**
	 * Tarea asncrona para descargar ciudades hdt.
	 *
	 * @author Elena Hervalejo
	 */
	private class DescargarCiudadTask extends HdtLittleDownloadTask<FicheroHdt, Boolean> {

		/** The notificador progreso. */
		private IProgress notificadorProgreso;

		/** The excep. */
		HdtException excep;

		/** The fichero ciudad. */
		FicheroHdt ficheroCiudad;

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#doInBackground(java.lang.Object[])
		 */
		@Override
		protected Boolean doInBackground(FicheroHdt... params) {
			try {
				this.ficheroCiudad = params[0];
				File fc = this.ficheroCiudad.getFichero();
				if (!fc.exists()) {
					this.publishProgress();

					File dir = FicherosService.obtenDirectorioHdts();

					String urlServer = PreferencesService.getHdtsUrl();

					String fileName = this.ficheroCiudad.getFichero().getName();

					this.downLoadFileFromUrl(urlServer, fileName, dir, fileName);

					FicherosService.guardaFicheroDescargado(this.ficheroCiudad);
				}

			} catch (HdtException e1) {
				this.excep = e1;
				return false;
			}
			return true;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#onCancelled()
		 */
		@Override
		protected void onCancelled() {
			this.notificadorProgreso.setFinProceso();
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#onPostExecute(java.lang.Object)
		 */
		@Override
		protected void onPostExecute(Boolean result) {
			if (result) {
				CiudadesListaPresenter.this.getFragment().refresh();
				this.notificadorProgreso.setFinProceso();

				CiudadesListaPresenter.this.openCiudadActivity(this.ficheroCiudad);
			} else {
				DialogAlerta.mostrarError(this.excep);
			}

		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#onPreExecute()
		 */
		@Override
		protected void onPreExecute() {
			this.notificadorProgreso = new BarProgressDialog(CiudadesListaPresenter.this.getActivity());
			this.notificadorProgreso.setTexto(HdTIdioma.getString(R.string.progreso_obteniendo_hdt));
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#onProgressUpdate(java.lang.Object[])
		 */
		@Override
		protected void onProgressUpdate(Integer... values) {
			if (values.length > 0) {
				this.notificadorProgreso.setProgresoProceso(values[0]);
			} else {
				this.notificadorProgreso.setTitulo(HdTIdioma.getString(R.string.progreso_obteniendo_hdt_title, this.ficheroCiudad.getNombre()));
				this.notificadorProgreso.setInicioProceso();
			}
		}
	}

	/** The descarga ciudad task. */
	private DescargarCiudadTask descargaCiudadTask;

	/**
	 * Mtodo de descarga de una ciudad hdt de forma asncrona.
	 *
	 * @param ciudad
	 *            the ciudad
	 * @return true, if successful
	 */
	public boolean descargaCiudad(FicheroHdt ciudad) {
		this.descargaCiudadTask = new DescargarCiudadTask();
		this.descargaCiudadTask.execute(ciudad);
		return true;
	}

	/**
	 * On destroy.
	 */
	public void onDestroy() {
		this.descargaCiudadTask.cancel(true);
	}

	/**
	 * Abre la pantalla de informacin de una ciudad.
	 *
	 * @param ciudad
	 *            the ciudad
	 */
	public void openCiudadActivity(FicheroHdt ciudad) {
		Intent mainIntent = new Intent(CiudadesListaPresenter.this.getActivity(), CiudadActivity.class);

		mainIntent.putExtra("CIUDAD", ciudad);
		CiudadesListaPresenter.this.getActivity().startActivityForResult(mainIntent, 0000);
	}

	/**
	 * Obtiene la actividad que contiene la lista de ciudades o que contiene el fragment.
	 *
	 * @return the activity
	 */
	protected abstract BaseActionBarActivity getActivity();

	/**
	 * Obtiene el fragment que contiene la lista de ciudades.
	 *
	 * @return the fragment
	 */
	protected abstract BaseFragment getFragment();
}
