/*
 * @author Elena Hervalejo
 */
package org.hdtourist.presenter;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.hdtourist.R;
import org.hdtourist.model.Ciudad;
import org.hdtourist.model.FicheroHdt;
import org.hdtourist.model.services.FicherosService;
import org.hdtourist.model.services.HdTouristService;
import org.hdtourist.model.services.PreferencesService;
import org.hdtourist.model.services.xml.FicherosConfigParser;
import org.hdtourist.utils.HdtException;
import org.hdtourist.utils.HdtFilter;
import org.hdtourist.view.utils.HdTIdioma;

import android.os.AsyncTask;
import android.util.Log;

// TODO: Auto-generated Javadoc
/**
 * Clase que extiende de AsyncTask y contiene utileras para la conexin breve a Internet.
 *
 * <b> HdtTask's generic types</b> AsyncTask utiliza 3 tipos, pero en el caso de HdtTask, se utilizarn dos, debido a que el de publicacin del progreso siempre ser de tipo Integer <li>
 * <ul>
 * Params, el tipo del parmetro enviado a la tarea a travs de la ejecucin.
 * </ul>
 * <ul>
 * Progres, el tipo del resultado del proceso
 * </ul>
 * </li>
 *
 * @author Elena Hervalejo
 * @param <Params>
 *            the generic type
 * @param <Result>
 *            the generic type
 */
public abstract class HdtLittleDownloadTask<Params, Result> extends AsyncTask<Params, Integer, Result> {

	/** La constante LOG_TAG. */
	private static final String LOG_TAG = "HdtTask";

	/**
	 * Obtiene el tamao del archivo indicado, realiza una conexin para ello.
	 *
	 * @param urlServer
	 *            String, direccin del servidor donde se aloja el archivo
	 * @param fileName
	 *            String, nombre del archiv en el servidor
	 * @return the file size
	 */
	public BigDecimal getFileSize(String urlServer, String fileName) {

		try {
			URL url = new URL(urlServer + fileName);

			HttpURLConnection ucon = (HttpURLConnection) url.openConnection();
			ucon.connect();
			if (ucon.getResponseCode() > 299 || ucon.getResponseCode() < 200) {
				return BigDecimal.ZERO;
			}
			final String contentLengthStr = ucon.getHeaderField("content-length");
			if (contentLengthStr != null) {
				return new BigDecimal(contentLengthStr);
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * Obtiene la lista de todos los ficheros HDT que se encuentran en el servidor configurado.
	 *
	 * @return lista de ficheros disponibles en el servidor
	 * @throws HdtException
	 *             Excepcin de la aplicacin
	 */
	public ArrayList<FicheroHdt> obtenFicherosHdtServidor() throws HdtException {
		ArrayList<FicheroHdt> result = new ArrayList<FicheroHdt>();

		if (!FicherosService.comprobarMemoria()) {
			throw new HdtException(HdTIdioma.getString(R.string.mensaje_error_memoria));
		}

		String urlServer = PreferencesService.getHdtsUrl();
		String nombreFicheroServer = HdTouristService.getInstance().getContexto().getResources().getString(R.string.const_server_ciudades_xml);

		File dir = FicherosService.getInstance().obtenDirectorioConfiguracion();

		this.downLoadFileFromUrl(urlServer, nombreFicheroServer, dir, nombreFicheroServer);

		File ficheroXmlCiudadesServidor = new File(dir.getAbsolutePath() + File.separator + nombreFicheroServer);

		File ruta_sd = FicherosService.getInstance().obtenDirectorioConfiguracion();
		String[] s = ruta_sd.list(new HdtFilter());
		if (s == null) {
			return result;
		}
		FicherosConfigParser configParser = FicherosConfigParser.getInstance();
		configParser.setFile(ficheroXmlCiudadesServidor);
		List<Ciudad> ficheros = configParser.obtenerFicheros();
		Iterator<Ciudad> i = ficheros.iterator();
		while (i.hasNext()) {
			Ciudad ficheroConfig = i.next();
			FicheroHdt f = new FicheroHdt();
			f.setRutaFichero(ruta_sd + File.separator + ficheroConfig.getNombre());
			f.setNombre(ficheroConfig.getNombreCiudad());
			f.setUri(ficheroConfig.getUri());
			f.setTamano(ficheroConfig.getSize());
			f.setTamanoMaps(ficheroConfig.getSizeMaps());
			result.add(f);
		}
		return result;
	}

	/**
	 * Mtodo para descargar ficheros pequeos de internet.
	 *
	 * @param fileURL
	 *            String con la direccin del servidor
	 * @param fileName
	 *            String con el nombre (en el servidor) del archivo a descargar
	 * @param newFileDir
	 *            File, directorio donde queremos guardar el archivo, debe existir
	 * @param newFileName
	 *            String con el nombre con el que queremos guardar el archivo
	 * @throws HdtException
	 *             Excepcin si la descarga no se realiza correctamente
	 */
	protected void downLoadFileFromUrl(String fileURL, String fileName, File newFileDir, String newFileName) throws HdtException { // this is the downloader method
		File file = new File(newFileDir, newFileName);
		try {

			BigDecimal totalSize = this.getFileSize(fileURL, fileName);
			BigDecimal downloadedSize = BigDecimal.ZERO;

			URL url = new URL(fileURL + fileName);
			file.createNewFile();
			long startTime = System.currentTimeMillis();
			Log.d(HdtLittleDownloadTask.LOG_TAG, "download begining");
			Log.d(HdtLittleDownloadTask.LOG_TAG, "download url:" + url);
			Log.d(HdtLittleDownloadTask.LOG_TAG, "downloaded file size:" + totalSize);
			/* Open a connection to that URL. */
			URLConnection ucon = url.openConnection();

			/*
			 * Define InputStreams to read from the URLConnection.
			 */
			InputStream is = ucon.getInputStream();

			/*
			 * Read bytes to the Buffer until there is nothing more to read(-1) and write on the fly in the file.
			 */
			FileOutputStream fos = new FileOutputStream(file);
			final int BUFFER_SIZE = 23 * 1024;
			BufferedInputStream bis = new BufferedInputStream(is, BUFFER_SIZE);
			byte[] baf = new byte[BUFFER_SIZE];
			int actual = 0;
			while (actual != -1) {
				fos.write(baf, 0, actual);
				actual = bis.read(baf, 0, BUFFER_SIZE);

				if (totalSize != null) {
					downloadedSize = BigDecimal.valueOf(file.length());
					BigDecimal prop = downloadedSize.divide(totalSize, 2, RoundingMode.FLOOR).multiply(BigDecimal.valueOf(100d));
					Log.v(HdtLittleDownloadTask.LOG_TAG, "downloaded: " + prop.toString());

					this.publishProgress(prop.intValue());
				}
			}

			fos.close();
			Log.d(HdtLittleDownloadTask.LOG_TAG, "download ready in" + (System.currentTimeMillis() - startTime) / 1000 + " sec");

		} catch (IOException e) {
			throw new HdtException(HdTIdioma.getString(R.string.mensaje_error_descarga));
		}

	}

}
