/*
 * @author Elena Hervalejo
 */
package org.hdtourist.presenter;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipInputStream;

import org.hdtourist.R;
import org.hdtourist.model.services.FicherosService;
import org.hdtourist.utils.HdtException;
import org.hdtourist.view.utils.HdTIdioma;

import android.os.AsyncTask;
import android.util.Log;

// TODO: Auto-generated Javadoc
/**
 * Clase que extiende de AsyncTask y contiene utileras para descomprimir archivos.
 *
 * <b> HdtTask's generic types</b> AsyncTask utiliza 3 tipos, pero en el caso de HdtTask, se utilizarn dos, debido a que el de publicacin del progreso siempre ser de tipo Integer <li>
 * <ul>
 * Params, el tipo del parmetro enviado a la tarea a travs de la ejecucin.
 * </ul>
 * <ul>
 * Progres, el tipo del resultado del proceso
 * </ul>
 * </li>
 *
 * @author Elena Hervalejo
 * @param <Params>
 *            the generic type
 * @param <Result>
 *            the generic type
 */
public abstract class HdtUnzipTask<Params, Result> extends AsyncTask<Params, Integer, Result> {

	/** La constante LOG_TAG. */
	private static final String LOG_TAG = "HdtUnzipTask";

	/**
	 * Descomprime un fichero zip, sobreescribiendo los ficheros existentes y permite notificar el progreso.
	 *
	 * @param zipFile
	 *            Ruta completa del fichero a descomprimir
	 * @param location
	 *            Ruta donde se descomprimir el directorio, si no existe se crea
	 * @throws HdtException
	 *             Excepcin de la aplicacin
	 */
	public void unzip(String zipFile, String location) throws HdtException {
		int size;
		byte[] buffer = new byte[FicherosService.BUFFER_SIZE];

		try {
			long startTime = System.currentTimeMillis();
			if (!location.endsWith(File.separator)) {
				location += File.separator;
			}
			File f = new File(location);
			if (!f.isDirectory()) {
				f.mkdirs();
			}
			ZipFile zip = new ZipFile(zipFile);
			int totalFiles = zip.size();
			int unzipFiles = 0;
			ZipInputStream zin = new ZipInputStream(new BufferedInputStream(new FileInputStream(zipFile), FicherosService.BUFFER_SIZE));
			try {
				ZipEntry ze = null;
				while ((ze = zin.getNextEntry()) != null) {
					String path = location + ze.getName();
					File unzipFile = new File(path);

					if (ze.isDirectory()) {
						if (!unzipFile.isDirectory()) {
							unzipFile.mkdirs();
						}
					} else {
						// check for and create parent directories if they don't
						// exist
						File parentDir = unzipFile.getParentFile();
						if (null != parentDir) {
							if (!parentDir.isDirectory()) {
								parentDir.mkdirs();
							}
						}

						// unzip the file
						FileOutputStream out = new FileOutputStream(unzipFile, false);
						BufferedOutputStream fout = new BufferedOutputStream(out, FicherosService.BUFFER_SIZE);
						try {
							while ((size = zin.read(buffer, 0, FicherosService.BUFFER_SIZE)) != -1) {
								fout.write(buffer, 0, size);
							}

							zin.closeEntry();
						} finally {
							fout.flush();
							fout.close();
							unzipFiles++;
							Log.v(HdtUnzipTask.LOG_TAG, "unzip: " + 100 * unzipFiles / totalFiles);

							this.publishProgress(100 * unzipFiles / totalFiles);
						}
					}
				}
			} finally {
				zin.close();
			}

			Log.d(HdtUnzipTask.LOG_TAG, "unzip ready in" + (System.currentTimeMillis() - startTime) / 1000 + " sec");

		} catch (Exception e) {
			Log.e(FicherosService.class.toString(), "Unzip exception", e);
			throw new HdtException(HdTIdioma.getString(R.string.mensaje_error_descomprimiendo));
		}
	}

}
