/*
 * @author Elena Hervalejo
 */
package org.hdtourist.view;

import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

import org.hdtourist.R;
import org.hdtourist.model.GeoPosicionEtiqueta;
import org.hdtourist.model.ListaGeoPosiciones;
import org.hdtourist.model.services.HdTouristService;
import org.hdtourist.utils.HdtException;
import org.hdtourist.view.base.BaseActionBarActivity;
import org.hdtourist.view.utils.LocalizacionView;
import org.hdtourist.view.utils.MapEventListener;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Color;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.view.View;
import android.view.Window;
import android.widget.ZoomControls;

import com.nutiteq.MapView;
import com.nutiteq.components.Components;
import com.nutiteq.datasources.raster.StoredRasterDataSource;
import com.nutiteq.geometry.Point;
import com.nutiteq.log.Log;
import com.nutiteq.projections.EPSG3857;
import com.nutiteq.rasterlayers.RasterLayer;
import com.nutiteq.style.PointStyle;
import com.nutiteq.style.StyleSet;
import com.nutiteq.ui.DefaultLabel;
import com.nutiteq.utils.UnscaledBitmapLoader;
import com.nutiteq.vectorlayers.GeometryLayer;

/**
 * Activity para mostrar los mapas.
 */
public class MapaActivity extends BaseActionBarActivity {

	/** The base layer. */
	private RasterLayer baseLayer;

	/** La localizacin actual obtenida por GPS. */
	private Location locationActual;

	/** The location layer. */
	private GeometryLayer locationLayer;

	/** El listener para el GPS. */
	private LocationListener locationListener;

	/** El timer para la lectura de la posicin. */
	private Timer locationTimer;

	/** El Map View. */
	private MapView mapView;

	/** The point style. */
	private PointStyle pointStyle;

	/** The punto. */
	protected Point punto;

	/** The geo layer. */
	GeometryLayer geoLayer;

	/** The veces visible. */
	int vecesVisible = 0;
	// it is suggested to start and stop mapping in Activity lifecycle methods,
	// as following:
	/** The visible point. */
	boolean visiblePoint = false;

	/**
	 * Deinit gps.
	 */
	protected void deinitGps() {
		// remove listeners from location manager - otherwise we will leak memory
		LocationManager locationManager = (LocationManager) this.getSystemService(Context.LOCATION_SERVICE);
		locationManager.removeUpdates(this.locationListener);
	}

	/**
	 * Inicia el gps.
	 *
	 * @param locationCircle
	 *            the location circle
	 */
	protected void initGps(final LocalizacionView locationCircle) {

		this.locationListener = new LocationListener() {

			@Override
			public void onLocationChanged(Location location) {
				MapaActivity.this.locationActual = location;
			}

			@Override
			public void onProviderDisabled(String provider) {
				Log.debug("GPS onProviderDisabled");
			}

			@Override
			public void onProviderEnabled(String provider) {
				Log.debug("GPS onProviderEnabled");
			}

			@Override
			public void onStatusChanged(String provider, int status, Bundle extras) {
				Log.debug("GPS onStatusChanged " + provider + " to " + status);
			}
		};

		LocationManager locationManager = (LocationManager) this.getSystemService(Context.LOCATION_SERVICE);
		List<String> providers = locationManager.getProviders(true);
		for (String provider : providers) {
			locationManager.requestLocationUpdates(provider, 10000, 100, this.locationListener);
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.hdtourist.view.base.BaseActionBarActivity#onCreate(android.os.Bundle)
	 */
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		this.requestWindowFeature(Window.FEATURE_NO_TITLE);
		this.setContentView(R.layout.activity_map);

		Bundle bundle = this.getIntent().getExtras();
		ListaGeoPosiciones lista = (ListaGeoPosiciones) bundle.get("LISTA");

		this.setTitle(lista.getFichero().getNombre());

		this.mapView = (MapView) this.findViewById(R.id.act_mapa_mapview);
		// Optional, but very useful: restore map state during device rotation,
		// it is saved in onRetainNonConfigurationInstance() below
		Components retainObject = (Components) this.getLastNonConfigurationInstance();
		if (retainObject != null) {
			// just restore configuration, skip other initializations
			this.mapView.setComponents(retainObject);
			return;
		} else {
			// 2. create and set MapView components - mandatory
			Components components = new Components();
			this.mapView.setComponents(components);
		}

		String ruta = "";
		try {
			ruta = lista.getFichero().getRutaMapa();
		} catch (HdtException e) {
			this.manejaError(e);
		}
		if (!ruta.endsWith("/")) {
			ruta += "/";
		}
		StoredRasterDataSource dataSource = new StoredRasterDataSource(new EPSG3857(), 64, 12, 17, lista.getFichero().getNombreSinEspacios(), ruta);
		this.baseLayer = new RasterLayer(dataSource, 0);
		this.mapView.getLayers().setBaseLayer(this.baseLayer);

		this.mapView.setFocusPoint(this.mapView.getLayers().getBaseLayer().getProjection()
				.fromWgs84(lista.getCentro().longitude, lista.getCentro().latitude));

		// Establezco la rotacin al Norte
		this.mapView.setMapRotation(0f);
		// Tilt default is 90 degrees for "normal" 2D map view, minimum allowed is 30 degrees.
		this.mapView.setTilt(90.0f);

		this.geoLayer = new GeometryLayer(this.baseLayer.getProjection());
		for (GeoPosicionEtiqueta geoPos : lista.getListaPuntos()) {
			Bitmap pointMarker = UnscaledBitmapLoader.decodeResource(this.getResources(), R.drawable.ic_maps_location);
			StyleSet<PointStyle> pointStyleSet = new StyleSet<PointStyle>();
			PointStyle pointStyle = PointStyle.builder().setBitmap(pointMarker).setSize(0.3f).setColor(Color.RED).build();
			pointStyleSet.setZoomStyle(12, pointStyle);
			this.geoLayer.add(new Point(this.baseLayer.getProjection().fromWgs84(geoPos.getPosicion().longitude, geoPos.getPosicion().latitude),
					new DefaultLabel(geoPos.getEtiqueta()), pointStyle, geoPos));

			this.mapView.getLayers().addLayer(this.geoLayer);
		}
		if (lista.numPosiciones() == 1) {
			this.mapView.setZoom(15.0f);
		} else {
			// Hay ms de un punto, por lo que pongo un zoom menor
			this.mapView.setZoom(14.0f);
			// Dejo que se puedan abrir
			MapEventListener mapListener = new MapEventListener(this, true);
			this.mapView.getOptions().setMapListener(mapListener);
		}
		// }

		// Aadimos el control del zoom
		ZoomControls zoomControls = (ZoomControls) this.findViewById(R.id.act_mapa_zoomcontrols);
		zoomControls.setOnZoomInClickListener(new View.OnClickListener() {
			@Override
			public void onClick(final View v) {
				MapaActivity.this.mapView.zoomIn();
			}
		});
		zoomControls.setOnZoomOutClickListener(new View.OnClickListener() {
			@Override
			public void onClick(final View v) {
				MapaActivity.this.mapView.zoomOut();
			}
		});

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.support.v4.app.FragmentActivity#onDestroy()
	 */
	@Override
	protected void onDestroy() {
		super.onDestroy();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.support.v4.app.FragmentActivity#onStart()
	 */
	@Override
	protected void onStart() {
		super.onStart();

		// 4. Start the map - mandatory.
		this.mapView.startMapping();

		// Create layer for location circle
		this.locationLayer = new GeometryLayer(this.mapView.getLayers().getBaseProjection());
		this.mapView.getComponents().layers.addLayer(this.locationLayer);

		// add GPS My Location functionality
		final LocalizacionView locationCircle = new LocalizacionView(this.locationLayer);
		this.initGps(locationCircle);

		Bitmap pointMarker = UnscaledBitmapLoader.decodeResource(HdTouristService.getInstance().getContexto().getResources(),
				R.drawable.ic_action_location_found);
		StyleSet<PointStyle> pointStyleSet = new StyleSet<PointStyle>();
		this.pointStyle = PointStyle.builder().setBitmap(pointMarker).setSize(0.2f).setColor(Color.BLUE).build();
		pointStyleSet.setZoomStyle(12, this.pointStyle);
		// Run animation
		this.locationTimer = new Timer();
		this.locationTimer.scheduleAtFixedRate(new TimerTask() {
			@Override
			public void run() {
				if (MapaActivity.this.locationActual != null) {
					if (MapaActivity.this.punto == null) {
						MapaActivity.this.punto = new Point(MapaActivity.this.baseLayer.getProjection().fromWgs84(
								MapaActivity.this.locationActual.getLongitude(), MapaActivity.this.locationActual.getLatitude()), null,
								MapaActivity.this.pointStyle, null);
						MapaActivity.this.punto.setVisible(MapaActivity.this.visiblePoint);
						MapaActivity.this.locationLayer.add(MapaActivity.this.punto);
					} else {
						MapaActivity.this.punto.setMapPos(MapaActivity.this.baseLayer.getProjection().fromWgs84(
								MapaActivity.this.locationActual.getLongitude(), MapaActivity.this.locationActual.getLatitude()));
						MapaActivity.this.punto.setVisible(MapaActivity.this.visiblePoint);
					}
					if (MapaActivity.this.visiblePoint) {
						if (MapaActivity.this.vecesVisible == 2) {
							MapaActivity.this.visiblePoint = !MapaActivity.this.visiblePoint;
							MapaActivity.this.vecesVisible = 0;
						}
						MapaActivity.this.vecesVisible++;
					} else {
						MapaActivity.this.visiblePoint = !MapaActivity.this.visiblePoint;
					}
				}

			}
		}, 1000, 500);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.support.v7.app.ActionBarActivity#onStop()
	 */
	@Override
	protected void onStop() {
		// Stop animation
		this.locationTimer.cancel();

		// Remove created layer
		this.mapView.getComponents().layers.removeLayer(this.locationLayer);

		// remove GPS support, otherwise we will leak memory
		this.deinitGps();

		// Note: it is recommended to move startMapping() call to onStart method and implement onStop method (call MapView.stopMapping() from onStop).
		this.mapView.stopMapping();

		super.onStop();
	}
}