/*
 * @author Elena Hervalejo
 */
package org.hdtourist.view.utils;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.hdtourist.R;
import org.hdtourist.model.FicheroHdt;
import org.hdtourist.model.services.FicherosService;
import org.hdtourist.model.services.HdTouristService;
import org.hdtourist.model.services.PreferencesService;
import org.hdtourist.presenter.CiudadesServidorPresenter;
import org.hdtourist.utils.StringUtils;
import org.hdtourist.view.ObtenerCiudadesListFragment;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.TextView;

// TODO: Auto-generated Javadoc
/**
 * Adapter para la lista de ciudades en el servidor Construye los items View asocindo el listener para descargar o actualizar el hdt.
 *
 * @author Elena Hervalejo
 */
public class ListaObtenerCiudadesAdapter extends BaseAdapter {

	/** The busqueda. */
	private String busqueda;

	/** The fragment. */
	private final ObtenerCiudadesListFragment fragment;

	/** The lista. */
	private final List<FicheroHdt> lista;

	/** The presenter. */
	private CiudadesServidorPresenter presenter;

	/** Listener para actualizar o descargar un hdt. */
	View.OnClickListener clickListenerSeleccion = new View.OnClickListener() {

		@Override
		public void onClick(View view) {
			FicheroHdt elegido = (FicheroHdt) view.getTag();

			File fc = new File(elegido.getRutaFichero());
			if (!fc.exists()) {
				ListaObtenerCiudadesAdapter.this.confirmarYDescargarHdt(elegido);

			} else {
				ListaObtenerCiudadesAdapter.this.confirmarYActualizarHdt(elegido);
			}
		}
	};

	/**
	 * Constructor.
	 *
	 * @param ciudadesServidorPresenter
	 *            Presentador de la arquitectura M-V-P, para manejar la tarea de descargar hdt.
	 * @param lista
	 *            Lista de ciudades inicial, si hay una bsqueda ser filtrada
	 * @param ciudadesDisponiblesListFragment
	 *            listFragment que contendr la lista
	 * @param busqueda
	 *            si se enva vacio se mostrar toda la lista de ciudades del servidor, en caso contrario, se mostrarn aquellas ciduades que coincidan con la bsqueda
	 */
	public ListaObtenerCiudadesAdapter(CiudadesServidorPresenter ciudadesServidorPresenter, List<FicheroHdt> lista,
			ObtenerCiudadesListFragment ciudadesDisponiblesListFragment, String busqueda) {
		this.presenter = ciudadesServidorPresenter;
		this.lista = this.getListaResultado(lista, busqueda);
		this.fragment = ciudadesDisponiblesListFragment;
		this.busqueda = busqueda;
	}

	/**
	 * Comprueba la configuracin de la conexin antes de actualizar el hdt.
	 *
	 * @param elegido
	 *            the elegido
	 */
	public void confirmarYActualizarHdt(final FicheroHdt elegido) {
		AlertDialog.Builder builder = new AlertDialog.Builder(this.fragment.getActivity());

		builder.setMessage(HdTIdioma.getString(R.string.obtener_ciudades_already))
		.setTitle(HdTIdioma.getString(R.string.obtener_ciudades_already_title))
		.setPositiveButton(HdTIdioma.getString(R.string.dialog_ok_button), new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog, int id) {
				ListaObtenerCiudadesAdapter.this.descargarCiudad(elegido);
				dialog.cancel();
			}
		});
		builder.setNegativeButton(HdTIdioma.getString(R.string.dialog_cancelar_button), new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog, int which) {
				dialog.cancel();
			}
		});
		builder.create().show();
	}

	/**
	 * Comprueba la configuracin de la conexin antes de descargar el hdt.
	 *
	 * @param elegido
	 *            the elegido
	 */
	public void confirmarYDescargarHdt(final FicheroHdt elegido) {
		if (PreferencesService.isConexionNunca()) {
			DialogAlerta.mostrarAvisoPreferencias(this.fragment.getActivity(),
					HdTIdioma.getString(R.string.mensaje_aviso_conexion_desactivada_title),
					HdTIdioma.getString(R.string.mensaje_aviso_conexion_desactivada)).show();
			return;
		}

		this.descargarCiudad(elegido);
	}

	/**
	 * Lanza la tarea para descargar el hdt, comprobando si existe conexin en el dispositivo.
	 *
	 * @param elegido
	 *            the elegido
	 */
	public void descargarCiudad(FicheroHdt elegido) {
		Log.i("Descarga", "Descargando ciudad.");
		if (!HdTouristService.getInstance().isNetworkAvailable()) {
			DialogAlerta.mostrarAviso(this.fragment.getActivity(), HdTIdioma.getString(R.string.mensaje_error_no_conexion_title),
					HdTIdioma.getString(R.string.mensaje_error_no_conexion)).show();
			return;
		}
		this.presenter.descargaCiudad(elegido);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.widget.Adapter#getCount()
	 */
	@Override
	public int getCount() {
		return this.lista.size();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.widget.Adapter#getItem(int)
	 */
	@Override
	public Object getItem(int position) {
		return this.lista.get(position);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.widget.Adapter#getItemId(int)
	 */
	@Override
	public long getItemId(int position) {
		return position;
	}

	/**
	 * Obtiene la vista View, del elemento <tt>position</tt> de la lista.
	 *
	 * @param position
	 *            the position
	 * @param arg1
	 *            the arg1
	 * @param arg2
	 *            the arg2
	 * @return the view
	 */
	@Override
	public View getView(int position, View arg1, ViewGroup arg2) {
		LayoutInflater inflater = this.fragment.getActivity().getLayoutInflater();
		View view = inflater.inflate(R.layout.item_obtener_ciudad, null, true);
		TextView textView = (TextView) view.findViewById(R.id.act_obtener_ciudad_item_nombre);
		TextView sizeTextView = (TextView) view.findViewById(R.id.act_obtener_ciudad_item_size);
		ImageView imageAlready = (ImageView) view.findViewById(R.id.act_obtener_ciudad_item_already);
		FicheroHdt fichero = this.lista.get(position);
		textView.setText(fichero.getNombre());

		if (fichero.getFichero().exists()) {
			imageAlready.setVisibility(View.VISIBLE);
			sizeTextView.setVisibility(View.GONE);
		} else {
			sizeTextView.setVisibility(View.VISIBLE);
			sizeTextView.setText(FicherosService.getStringSize(fichero.getTamano()));
			imageAlready.setVisibility(View.GONE);
		}

		textView.setOnClickListener(this.clickListenerSeleccion);
		textView.setTag(fichero);

		return view;
	}

	/**
	 * Indica si hay resultados.
	 *
	 * @return <tt>true</tt> si no se est realizando una bsqueda o existen resultados de la bsqueda. <tt>false</tt> en caso contrario.
	 */
	public boolean haveResults() {
		return StringUtils.isEmpty(this.busqueda) || this.lista.size() > 0;
	}

	/**
	 * Obtiene la lista de ciudades que encaja con la bsqueda.
	 *
	 * @param lista
	 *            the lista
	 * @param busqueda
	 *            the busqueda
	 * @return the lista resultado
	 */
	private List<FicheroHdt> getListaResultado(List<FicheroHdt> lista, String busqueda) {
		if (StringUtils.isEmpty(busqueda)) {
			return lista;
		}
		List<FicheroHdt> listaResultado = new ArrayList<FicheroHdt>();
		for (FicheroHdt fichero : lista) {
			if (fichero.getNombre().toLowerCase().indexOf(busqueda.toLowerCase()) >= 0) {
				listaResultado.add(fichero);
			}
		}
		return listaResultado;
	}
}
